from datetime import datetime
from typing import Optional

import discord
from cpl_core.database import TableABC

from bot_data.model.scheduled_event_interval_enum import ScheduledEventIntervalEnum
from bot_data.model.server import Server


class ScheduledEvent(TableABC):
    def __init__(
        self,
        interval: ScheduledEventIntervalEnum,
        name: str,
        description: str,
        channel_id: int,
        start_time: datetime,
        end_time: Optional[datetime],
        entity_type: discord.EntityType,
        location: Optional[str],
        server: Optional[Server],
        created_at: datetime = None,
        modified_at: datetime = None,
        id=0,
    ):
        self._id = id
        self._interval = interval
        self._name = name
        self._description = description
        self._channel_id = channel_id
        self._start_time = start_time
        self._end_time = end_time
        self._entity_type = entity_type
        self._location = location
        self._server = server

        TableABC.__init__(self)
        self._created_at = created_at if created_at is not None else self._created_at
        self._modified_at = modified_at if modified_at is not None else self._modified_at

    @property
    def id(self) -> int:
        return self._id

    @property
    def interval(self) -> ScheduledEventIntervalEnum:
        return self._interval

    @interval.setter
    def interval(self, value: ScheduledEventIntervalEnum):
        self._interval = value

    @property
    def name(self) -> str:
        return self._name

    @name.setter
    def name(self, value: str):
        self._name = value

    @property
    def description(self) -> str:
        return self._description

    @description.setter
    def description(self, value: str):
        self._description = value

    @property
    def channel_id(self) -> int:
        return self._channel_id

    @channel_id.setter
    def channel_id(self, value: int):
        self._channel_id = value

    @property
    def start_time(self) -> datetime:
        return self._start_time

    @start_time.setter
    def start_time(self, value: datetime):
        self._start_time = value

    @property
    def end_time(self) -> datetime:
        return self._end_time

    @end_time.setter
    def end_time(self, value: datetime):
        self._end_time = value

    @property
    def entity_type(self) -> discord.EntityType:
        return self._entity_type

    @entity_type.setter
    def entity_type(self, value: discord.EntityType):
        self._entity_type = value

    @property
    def location(self) -> str:
        return self._location

    @location.setter
    def location(self, value: str):
        self._location = value

    @property
    def server(self) -> Server:
        return self._server

    @server.setter
    def server(self, value: Server):
        self._server = value

    @staticmethod
    def get_select_all_string() -> str:
        return str(
            f"""
            SELECT * FROM `ScheduledEvents`;
        """
        )

    @staticmethod
    def get_select_by_id_string(id: int) -> str:
        return str(
            f"""
            SELECT * FROM `ScheduledEvents`
            WHERE `Id` = {id};
        """
        )

    @staticmethod
    def get_select_by_server_id_string(s_id: int) -> str:
        return str(
            f"""
            SELECT * FROM `ScheduledEvents`
            WHERE `ServerId` = {s_id};
        """
        )

    @property
    def insert_string(self) -> str:
        return str(
            f"""
            INSERT INTO `ScheduledEvents` (
                `Interval`, `Name`, `Description`, `ChannelId`, `StartTime`, `EndTime`, `EntityType`, `Location`, `ServerId`
            ) VALUES (
                '{self._interval.value}',
                '{self._name}',
                {"NULL" if self._description is None else f"'{self._description}'"},
                {"NULL" if self._channel_id is None else f"'{self._channel_id}'"},
                '{self._start_time}',
                {"NULL" if self._end_time is None else f"'{self._end_time}'"},
                '{self._entity_type.value}',
                {"NULL" if self._location is None else f"'{self._location}'"},
                {self._server.id}
            );
        """
        )

    @property
    def udpate_string(self) -> str:
        return str(
            f"""
            UPDATE `ScheduledEvents`
            SET `Interval` = '{self._interval.value}', 
            `Name` = '{self._name}',
            `Description` = {"NULL" if self._description is None else f"'{self._description}'"},
            `ChannelId` = {"NULL" if self._channel_id is None else f"'{self._channel_id}'"},
            `StartTime` = '{self._start_time}',
            `EndTime` = {"NULL" if self._end_time is None else f"'{self._end_time}'"}, 
            `EntityType` = '{self._entity_type.value}',
            `Location` = {"NULL" if self._location is None else f"'{self._location}'"}
            WHERE `Id` = {self._id};
        """
        )

    @property
    def delete_string(self) -> str:
        return str(
            f"""
            DELETE FROM `ScheduledEvents`
            WHERE `Id` = {self._id};
        """
        )
