import { Component, OnInit } from "@angular/core";
import { ActivatedRoute, Router } from "@angular/router";
import { Server } from "src/app/models/data/server.model";
import { DataService } from "src/app/services/data/data.service";
import { SpinnerService } from "src/app/services/spinner/spinner.service";
import { SidebarService } from "../../../../services/sidebar/sidebar.service";
import { ServerService } from "../../../../services/server.service";
import { Query } from "../../../../models/graphql/query.model";
import { Queries } from "../../../../models/graphql/queries.model";
import { catchError } from "rxjs/operators";
import { throwError } from "rxjs";

@Component({
  selector: "app-server-dashboard",
  templateUrl: "./server-dashboard.component.html",
  styleUrls: ["./server-dashboard.component.scss"]
})
export class ServerDashboardComponent implements OnInit {

  id!: number;
  server!: Server;

  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private data: DataService,
    private spinner: SpinnerService,
    private serverService: ServerService
  ) {
  }

  ngOnInit(): void {
    this.data.getServerFromRoute(this.route).then(server => {
      // this.server = server;

      this.loadServer(server.id);
    });

    this.serverService.server$.subscribe(server => {
      if (!server) {
        return;
      }
      this.loadServer(server.id);
    });
  }

  loadServer(id?: number): void {
    this.data.query<Query>(Queries.serversDashboardQuery, {
      filter: {
        id: id
      }
    }).pipe(catchError(err => {
      this.spinner.hideSpinner();
      return throwError(() => err);
    })).subscribe(data => {
      this.server = data.servers[0];
      this.spinner.hideSpinner();
    });
  }

}
